<?php
namespace Fliglio\Fltk;

use Fliglio\Flfc\ResponseContent;
use Fliglio\Flfc\HasHeadersToSet;
use Fliglio\Flfc\Response;

class JsonView implements ResponseContent, HasHeadersToSet {
	
	protected $data;
	protected $encoded;
	
	public function __construct($data, $encoded=false) {
		$this->data    = $data;
		$this->encoded = $encoded;
	}
	
	public function setHeadersOnResponse(Response $response) {
		$response->addHeader('Content-Type', 'text/json');
	}
	
	public function render() {
		return $this->encoded ? $this->data : json_encode($this->data);
	}

	public function toDebugString() {
		return $this->indent($this->render());
	}
	
	/**
	 * http://recursive-design.com/blog/2008/03/11/format-json-with-php/
	 * 
	 * Indents a flat JSON string to make it more human-readable.
	 * 
	 * @param string $json The original JSON string to process.
	 * @return string Indented version of the original JSON string.
	 */
	private function indent($json) {

		$result      = '';
		$pos         = 0;
		$strLen      = strlen($json);
		$indentStr   = '  ';
		$newLine     = "\n";
		$prevChar    = '';
		$outOfQuotes = true;

		for ($i=0; $i<=$strLen; $i++) {

			// Grab the next character in the string.
			$char = substr($json, $i, 1);

			// Are we inside a quoted string?
			if ($char == '"' && $prevChar != '\\') {
				$outOfQuotes = !$outOfQuotes;

				// If this character is the end of an element, 
				// output a new line and indent the next line.
			} else if(($char == '}' || $char == ']') && $outOfQuotes) {
				$result .= $newLine;
				$pos --;
				for ($j=0; $j<$pos; $j++) {
					$result .= $indentStr;
				}
			}

			// Add the character to the result string.
			$result .= $char;

			// If the last character was the beginning of an element, 
			// output a new line and indent the next line.
			if (($char == ',' || $char == '{' || $char == '[') && $outOfQuotes) {
				$result .= $newLine;
				if ($char == '{' || $char == '[') {
					$pos ++;
				}

				for ($j = 0; $j < $pos; $j++) {
					$result .= $indentStr;
				}
			}

			$prevChar = $char;
		}

		return $result;
	}
}